@@ -37,6 +37,7 @@
 #include <gtk/gtk.h>
 #include <libxfce4util/libxfce4util.h>
 
+#include "cycle.h"
 #include "client.h"
 #include "focus.h"
 #include "frame.h"
@@ -51,12 +52,106 @@
 typedef struct _ClientCycleData ClientCycleData;
 struct _ClientCycleData
 {
-    Client *c;
     Tabwin *tabwin;
     Window wireframe;
-    int cycle_range;
 };
 
+#if 0
+static gint
+clientCompareApp (gconstpointer a, gconstpointer b)
+{
+    return !clientSameApplication ((Client *) a, (Client *) b);
+}
+#endif
+
+static guint
+clientGetCycleRange (ScreenInfo *screen_info)
+{
+    guint range;
+
+    g_return_val_if_fail (screen_info != NULL, 0);
+    TRACE ("entering clientGetCycleRange");
+
+    range = 0;
+    if (screen_info->params->cycle_hidden)
+    {
+        range |= SEARCH_INCLUDE_HIDDEN;
+    }
+    if (!screen_info->params->cycle_minimum)
+    {
+        range |= SEARCH_INCLUDE_SKIP_TASKBAR | SEARCH_INCLUDE_SKIP_PAGER;
+    }
+    if (screen_info->params->cycle_workspaces)
+    {
+        range |= SEARCH_INCLUDE_ALL_WORKSPACES;
+    }
+
+    return range;
+}
+
+static GList *
+clientCycleCreateList (Client *c)
+{
+    ScreenInfo *screen_info;
+    Client *c2;
+    guint range, i;
+    GList *client_list;
+
+    g_return_val_if_fail (c, NULL);
+    TRACE ("entering clientCycleCreateList");
+
+    screen_info = c->screen_info;
+    range = clientGetCycleRange (screen_info);
+    client_list = NULL;
+
+    for (c2 = c, i = 0; c && i < screen_info->client_count; i++, c2 = c2->next)
+    {
+        if (!clientSelectMask (c2, NULL, range,
+             screen_info->params->cycle_apps_only ? WINDOW_NORMAL : WINDOW_REGULAR_FOCUSABLE))
+            continue;
+#if 0
+        if (screen_info->params->cycle_apps_only)
+        {
+            /*
+             *  For apps only cycling, it's a tad more complicated
+             * - We want only regular windows (no dialog or anything else)
+             * - We don't want a window from the same application to be
+             *   in the list.
+             */
+            if (c2->type != WINDOW_NORMAL)
+                continue;
+            if (g_list_find_custom (client_list, c2, clientCompareApp))
+                continue;
+        }
+#endif
+        TRACE ("clientCycleCreateList: adding %s", c2->name);
+        client_list = g_list_append (client_list, c2);
+    }
+
+    return client_list;
+}
+
+static void
+clientCycleFocusAndRaise (Client *c)
+{
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+    Client *sibling;
+
+    g_return_if_fail (c != NULL);
+    TRACE ("entering clientFocusAndRaise");
+
+    screen_info = c->screen_info;
+    display_info = screen_info->display_info;
+
+    sibling = clientGetTransientFor(c);
+    clientRaise (sibling, None);
+    clientShow (sibling, TRUE);
+    clientUnshade (c);
+    clientSetFocus (screen_info, c, myDisplayGetCurrentTime (display_info), NO_FOCUS_FLAG);
+    clientSetLastRaise (c);
+}
+
 static eventFilterStatus
 clientCycleEventFilter (XEvent * xevent, gpointer data)
 {
@@ -72,17 +167,18 @@ clientCycleEventFilter (XEvent * xevent, gpointer data)
     TRACE ("entering clientCycleEventFilter");
 
     passdata = (ClientCycleData *) data;
-    if (passdata->c == NULL)
+    c = tabwinGetSelected(passdata->tabwin);
+    if (c == NULL)
     {
         return EVENT_FILTER_CONTINUE;
     }
 
-    c = passdata->c;
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
     cancel = screen_info->params->keys[KEY_CANCEL].keycode;
     modifier = screen_info->params->keys[KEY_CYCLE_WINDOWS].modifier;
     status = EVENT_FILTER_STOP;
+    removed = NULL;
     cycling = TRUE;
     gone = FALSE;
 
@@ -92,18 +188,17 @@ clientCycleEventFilter (XEvent * xevent, gpointer data)
     switch (xevent->type)
     {
         case DestroyNotify:
-            removed = myScreenGetClientFromWindow (screen_info, ((XDestroyWindowEvent *) xevent)->window, SEARCH_WINDOW);
-            gone |= (c == removed);
-            c = tabwinRemoveClient(passdata->tabwin, removed);
-            passdata->c = c;
             status = EVENT_FILTER_CONTINUE;
+            if ((removed = myScreenGetClientFromWindow (screen_info, ((XDestroyWindowEvent *) xevent)->window, SEARCH_WINDOW)) == NULL)
+                break; /* No need to go any further */
+            gone |= (c == removed);
             /* Walk through */
         case UnmapNotify:
-            removed = myScreenGetClientFromWindow (screen_info, ((XUnmapEvent *) xevent)->window, SEARCH_WINDOW);
+            status = EVENT_FILTER_CONTINUE;
+            if (!removed && (removed = myScreenGetClientFromWindow (screen_info, ((XUnmapEvent *) xevent)->window, SEARCH_WINDOW)) == NULL)
+                break; /* No need to go any further */
             gone |= (c == removed);
             c = tabwinRemoveClient(passdata->tabwin, removed);
-            passdata->c = c;
-            status = EVENT_FILTER_CONTINUE;
             /* Walk through */
         case KeyPress:
             key_pressed = (xevent->type == KeyPress);
@@ -115,11 +210,11 @@ clientCycleEventFilter (XEvent * xevent, gpointer data)
                     key = myScreenGetKeyPressed (screen_info, (XKeyEvent *) xevent);
                     /*
                      * We cannot simply check for key == KEY_CANCEL here because of the
-                     * mofidier being pressed, so we need to look at the keycode directly.
+                     * modidier being pressed, so we need to look at the keycode directly.
                      */
                     if (xevent->xkey.keycode == cancel)
                     {
-                        c2 = tabwinGetHead (passdata->tabwin);
+                        c2 = tabwinSelectHead (passdata->tabwin);
                         cycling = FALSE;
                     }
                     else if (key == KEY_CYCLE_REVERSE_WINDOWS)
@@ -135,7 +230,6 @@ clientCycleEventFilter (XEvent * xevent, gpointer data)
                     if (c2)
                     {
                         c = c2;
-                        passdata->c = c;
                     }
 
                     /* If last key press event had not our modifier pressed, finish cycling */
@@ -197,6 +291,8 @@ clientCycle (Client * c, XKeyEvent * ev)
     ScreenInfo *screen_info;
     DisplayInfo *display_info;
     ClientCycleData passdata;
+    GList *client_list, *selected;
+    guint cycle_range;
     gboolean g1, g2;
     int key;
 
@@ -206,6 +302,12 @@ clientCycle (Client * c, XKeyEvent * ev)
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
 
+    client_list = clientCycleCreateList (c);
+    if (!client_list)
+    {
+        return;
+    }
+
     g1 = myScreenGrabKeyboard (screen_info, ev->time);
     g2 = myScreenGrabPointer (screen_info, LeaveWindowMask,  None, ev->time);
 
@@ -214,96 +316,124 @@ clientCycle (Client * c, XKeyEvent * ev)
         TRACE ("grab failed in clientCycle");
 
         gdk_beep ();
-        myScreenUngrabKeyboard (screen_info, ev->time);
-        myScreenUngrabPointer (screen_info, ev->time);
+        myScreenUngrabKeyboard (screen_info, myDisplayGetCurrentTime (display_info));
+        myScreenUngrabPointer (screen_info, myDisplayGetCurrentTime (display_info));
+        g_list_free (client_list);
 
         return;
     }
-
-    if (screen_info->params->cycle_hidden)
-    {
-        passdata.cycle_range = INCLUDE_HIDDEN;
-    }
-    else
-    {
-        passdata.cycle_range = 0;
-    }
-    if (!screen_info->params->cycle_minimum)
-    {
-        passdata.cycle_range |= INCLUDE_SKIP_TASKBAR | INCLUDE_SKIP_PAGER;
-    }
-    if (screen_info->params->cycle_workspaces)
-    {
-        passdata.cycle_range |= INCLUDE_ALL_WORKSPACES;
-    }
+    cycle_range = clientGetCycleRange (screen_info);
     key = myScreenGetKeyPressed (screen_info, ev);
+
     if (key == KEY_CYCLE_REVERSE_WINDOWS)
     {
-        passdata.c = clientGetPrevious (c, passdata.cycle_range);
+        selected = g_list_last (client_list);
     }
     else
     {
-        passdata.c = clientGetNext (c, passdata.cycle_range);
+        selected = g_list_next (client_list);
+    }
+    if (!selected)
+    {
+        /* Only one element in list */
+        selected = client_list;
     }
     passdata.wireframe = None;
 
-    /* If there is one single client, and if it's eligible for focus, use it */
-    if ((passdata.c == NULL) && (c != clientGetFocus()) &&
-        clientSelectMask (c, passdata.cycle_range, WINDOW_REGULAR_FOCUSABLE))
+    TRACE ("entering cycle loop");
+    if (screen_info->params->cycle_draw_frame)
     {
-        passdata.c = c;
+        passdata.wireframe = wireframeCreate ((Client *) selected->data);
     }
-
-    if (passdata.c)
+    passdata.tabwin = tabwinCreate (&client_list, selected, screen_info->params->cycle_workspaces);
+    eventFilterPush (display_info->xfilter, clientCycleEventFilter, &passdata);
+    gtk_main ();
+    eventFilterPop (display_info->xfilter);
+    TRACE ("leaving cycle loop");
+    if (passdata.wireframe)
     {
-        TRACE ("entering cycle loop");
-        if (screen_info->params->cycle_draw_frame)
-        {
-            passdata.wireframe = wireframeCreate (passdata.c);
-        }
-        passdata.tabwin = tabwinCreate (passdata.c->screen_info->gscr, c,
-                                        passdata.c, passdata.cycle_range,
-                                        screen_info->params->cycle_workspaces);
-        eventFilterPush (display_info->xfilter, clientCycleEventFilter, &passdata);
-        gtk_main ();
-        eventFilterPop (display_info->xfilter);
-        TRACE ("leaving cycle loop");
-        tabwinDestroy (passdata.tabwin);
-        g_free (passdata.tabwin);
-        if (passdata.wireframe)
-        {
-            wireframeDelete (screen_info, passdata.wireframe);
-        }
-        updateXserverTime (display_info);
+        wireframeDelete (screen_info, passdata.wireframe);
     }
+    updateXserverTime (display_info);
 
-    if (passdata.c)
+    c = tabwinGetSelected (passdata.tabwin);
+    if (c)
     {
         Client *focused;
-        Client *sibling;
-        int workspace;
+        guint workspace;
 
-        c = passdata.c;
         workspace = c->win_workspace;
         focused = clientGetFocus ();
 
+        if ((focused) && (c != focused))
+        {
+            /* We might be able to avoid this if we are about to switch workspace */
+            clientAdjustFullscreenLayer (focused, FALSE);
+        }
+        if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_WAS_SHOWN))
+        {
+            /* We are explicitely activating a window that was shown before show-desktop */
+            clientClearAllShowDesktop (screen_info);
+        }
         if (workspace != screen_info->current_ws)
         {
             workspaceSwitch (screen_info, workspace, c, FALSE, myDisplayGetCurrentTime (display_info));
         }
 
-        if ((focused) && (passdata.c != focused))
-        {
-            clientClearAllShowDesktop (screen_info);
-            clientAdjustFullscreenLayer (focused, FALSE);
-        }
-
-        sibling = clientGetTransientFor(c);
-        clientRaise (sibling, None);
-        clientShow (sibling, TRUE);
-        clientSetFocus (screen_info, c, myDisplayGetCurrentTime (display_info), NO_FOCUS_FLAG);
+        clientCycleFocusAndRaise (c);
     }
 
+    tabwinDestroy (passdata.tabwin);
+    g_free (passdata.tabwin);
+    g_list_free (client_list);
+
     myScreenUngrabKeyboard (screen_info, myDisplayGetCurrentTime (display_info));
     myScreenUngrabPointer (screen_info, myDisplayGetCurrentTime (display_info));
 }
+
+gboolean
+clientSwitchWindow (void)
+{
+    Client *focus, *new;
+    guint range;
+
+    focus = clientGetFocus();
+    if (!focus)
+    {
+        return FALSE;
+    }
+
+    range = clientGetCycleRange (focus->screen_info);
+    new = clientGetPrevious(focus, range | SEARCH_SAME_APPLICATION, WINDOW_REGULAR_FOCUSABLE);
+    if (new)
+    {
+        clientCycleFocusAndRaise (new);
+        return TRUE;
+    }
+    return FALSE;
+}
+
+gboolean
+clientSwitchApp (void)
+{
+    Client *focus, *new;
+    guint range;
+
+    TRACE ("entering clientSwitchApp");
+
+    focus = clientGetFocus();
+    if (!focus)
+    {
+        return FALSE;
+    }
+
+    range = clientGetCycleRange (focus->screen_info);
+    /* We do not want dialogs, just toplevel app windows here */
+    new = clientGetPrevious(focus, range | SEARCH_DIFFERENT_APPLICATION, WINDOW_NORMAL);
+    if (new)
+    {
+        clientCycleFocusAndRaise (new);
+        return TRUE;
+    }
+    return FALSE;
+}
