@@ -36,5 +36,6 @@
 
 void                     clientCycle                            (Client *,
                                                                  XKeyEvent *);
-
+gboolean                 clientSwitchWindow                     (void);
+gboolean                 clientSwitchApp                        (void);
 #endif /* INC_CYCLE_H */
