@@ -41,6 +41,7 @@
 #include <X11/extensions/Xrandr.h>
 #endif
 #include <libxfce4util/libxfce4util.h>
+#include <libxfce4ui/libxfce4ui.h>
 
 #include "misc.h"
 #include "workspaces.h"
@@ -252,22 +253,6 @@ typeOfClick (ScreenInfo *screen_info, Window w, XEvent * ev, gboolean allow_doub
     return (XfwmButtonClickType) passdata.clicks;
 }
 
-#if CHECK_BUTTON_TIME
-static gboolean
-check_button_time (XButtonEvent *ev)
-{
-    static guint32 last_button_time = CurrentTime;
-
-    if (last_button_time > ev->time)
-    {
-        return FALSE;
-    }
-
-    last_button_time = ev->time;
-    return TRUE;
-}
-#endif
-
 static void
 toggle_show_desktop (ScreenInfo *screen_info)
 {
@@ -291,6 +276,7 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
     eventFilterStatus status;
     ScreenInfo *screen_info;
     ScreenInfo *ev_screen_info;
+    gboolean handled;
     Client *c;
     int key;
 
@@ -299,12 +285,10 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
     ev_screen_info = myDisplayGetScreenFromRoot (display_info, ev->root);
     if (!ev_screen_info)
     {
-        /* Release queued events */
-        XAllowEvents (display_info->dpy, SyncKeyboard, CurrentTime);
-
         return EVENT_FILTER_PASS;
     }
 
+    handled = FALSE;
     status = EVENT_FILTER_PASS;
     c = clientGetFocus ();
     if (c)
@@ -395,17 +379,16 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
             case KEY_MOVE_WORKSPACE_10:
             case KEY_MOVE_WORKSPACE_11:
             case KEY_MOVE_WORKSPACE_12:
-                if (key - KEY_MOVE_WORKSPACE_1 < screen_info->workspace_count)
+                if ((guint) (key - KEY_MOVE_WORKSPACE_1) < screen_info->workspace_count)
                 {
                     clientRaise (c, None);
                     workspaceSwitch (screen_info, key - KEY_MOVE_WORKSPACE_1, c, TRUE, ev->time);
                 }
                 break;
             case KEY_POPUP_MENU:
-                XAllowEvents (display_info->dpy, SyncKeyboard, CurrentTime);
                 show_window_menu (c, frameX (c) + frameLeft (c),
                                      frameY (c) + frameTop (c),
-                                     Button1, GDK_CURRENT_TIME);
+                                     Button1, ev->time);
                 break;
             case KEY_FILL_WINDOW:
                 clientFill (c, CLIENT_FILL);
@@ -414,7 +397,7 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
                 clientFill (c, CLIENT_FILL_VERT);
                 break;
             case KEY_FILL_HORIZ:
-                clientFill (c, CLIENT_FILL_HORIZ);
+                    clientFill (c, CLIENT_FILL_HORIZ);
                 break;
             default:
                 break;
@@ -436,7 +419,7 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
                 status = EVENT_FILTER_REMOVE;
                 if (display_info->session)
                 {
-                    logout_session (display_info->session);
+                    xfce_sm_client_request_shutdown(display_info->session, XFCE_SM_CLIENT_SHUTDOWN_HINT_LOGOUT);
                 }
                 break;
             default:
@@ -446,6 +429,12 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
 
     switch (key)
     {
+        case KEY_SWITCH_WINDOW:
+            clientSwitchWindow ();
+            break;
+        case KEY_SWITCH_APPLICATION:
+            clientSwitchApp ();
+            break;
         case KEY_NEXT_WORKSPACE:
             status = EVENT_FILTER_REMOVE;
             workspaceSwitch (ev_screen_info, ev_screen_info->current_ws + 1, NULL, TRUE, ev->time);
@@ -497,7 +486,7 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
         case KEY_WORKSPACE_11:
         case KEY_WORKSPACE_12:
             status = EVENT_FILTER_REMOVE;
-            if (key - KEY_WORKSPACE_1 < ev_screen_info->workspace_count)
+            if ((guint) (key - KEY_WORKSPACE_1) < ev_screen_info->workspace_count)
             {
                 workspaceSwitch (ev_screen_info, key - KEY_WORKSPACE_1, NULL, TRUE, ev->time);
             }
@@ -510,9 +499,6 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
             break;
     }
 
-    /* Release queued events */
-    XAllowEvents (display_info->dpy, SyncKeyboard, CurrentTime);
-
     return status;
 }
 
@@ -520,9 +506,10 @@ static eventFilterStatus
 handleKeyRelease (DisplayInfo *display_info, XKeyEvent * ev)
 {
     TRACE ("entering handleKeyRelease");
-
-    /* Release queued events */
-    XAllowEvents (display_info->dpy, SyncKeyboard, CurrentTime);
+    /* Seems we loose KeyRelease as a results of active grabs after
+     * a workspace switch of a cycling.
+     */
+    XAllowEvents (display_info->dpy, ReplayKeyboard, ev->time);
 
     return EVENT_FILTER_PASS;
 }
@@ -536,7 +523,7 @@ static void
 edgeButton (Client * c, int part, XButtonEvent * ev)
 {
     ScreenInfo *screen_info;
-    int state;
+    guint state;
 
     screen_info = c->screen_info;
     state = ev->state & MODIFIER_MASK;
@@ -697,7 +684,7 @@ button1Action (Client * c, XButtonEvent * ev)
 }
 
 static void
-titleButton (Client * c, int state, XButtonEvent * ev)
+titleButton (Client * c, guint state, XButtonEvent * ev)
 {
     ScreenInfo *screen_info;
     DisplayInfo *display_info;
@@ -827,7 +814,7 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
     ScreenInfo *screen_info;
     Client *c;
     Window win;
-    int state, part;
+    guint state, part;
     gboolean replay;
 
     TRACE ("entering handleButtonPress");
@@ -840,19 +827,29 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
         win = ev->subwindow;
         screen_info = c->screen_info;
 
-        if ((ev->button == Button1) && (screen_info->params->easy_click) && (state == screen_info->params->easy_click))
+        if ((ev->button == Button1) && (state) && (state == screen_info->params->easy_click))
         {
             button1Action (c, ev);
         }
-        else if ((ev->button == Button2) && (screen_info->params->easy_click) && (state == screen_info->params->easy_click))
+        else if ((ev->button == Button2) && (state) && (state == screen_info->params->easy_click))
         {
             clientLower (c, None);
         }
-        else if ((ev->button == Button3) && (screen_info->params->easy_click) && (state == screen_info->params->easy_click))
+        else if ((ev->button == Button3) && (state) && (state == screen_info->params->easy_click))
         {
             part = edgeGetPart (c, ev);
             edgeButton (c, part, ev);
         }
+#if 0   /* Binding the alt+scroll wheel to switch app/window is not handy, disabling for now */
+        else if ((ev->button == Button4) && (state) && (state == screen_info->params->easy_click))
+        {
+            clientSwitchWindow ();
+        }
+        else if ((ev->button == Button5) && (state) && (state == screen_info->params->easy_click))
+        {
+            clientSwitchApp ();
+        }
+#endif
         else if (WIN_IS_BUTTON (win))
         {
             if (ev->button <= Button3)
@@ -996,7 +993,7 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
     }
 
     /* Release pending events */
-    XAllowEvents (display_info->dpy, replay ? ReplayPointer : SyncPointer, CurrentTime);
+    XAllowEvents (display_info->dpy, replay ? ReplayPointer : SyncPointer, myDisplayGetCurrentTime (display_info));
 
     return EVENT_FILTER_REMOVE;
 }
@@ -1016,7 +1013,7 @@ handleButtonRelease (DisplayInfo *display_info, XButtonEvent * ev)
     }
 
     /* Release pending events */
-    XAllowEvents (display_info->dpy, SyncPointer, CurrentTime);
+    XAllowEvents (display_info->dpy, SyncPointer, myDisplayGetCurrentTime (display_info));
 
     return EVENT_FILTER_REMOVE;
 }
@@ -1089,8 +1086,11 @@ handleMapRequest (DisplayInfo *display_info, XMapRequestEvent * ev)
             status = EVENT_FILTER_REMOVE;
         }
 
+        if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_WAS_SHOWN))
+        {
+             clientClearAllShowDesktop (screen_info);
+        }
         clientShow (c, TRUE);
-        clientClearAllShowDesktop (screen_info);
 
         if (FLAG_TEST (c->flags, CLIENT_FLAG_STICKY) ||
             (c->win_workspace == screen_info->current_ws))
@@ -1186,12 +1186,14 @@ handleUnmapNotify (DisplayInfo *display_info, XUnmapEvent * ev)
          */
         if ((ev->event == screen_info->xroot) && (ev->send_event))
         {
-            TRACE ("ICCCM UnmapNotify for \"%s\"", c->name);
-            list_of_windows = clientListTransientOrModal (c);
-            clientPassFocus (screen_info, c, list_of_windows);
-            clientUnframe (c, FALSE);
-            g_list_free (list_of_windows);
-
+            if (!FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE))
+            {
+                TRACE ("ICCCM UnmapNotify for \"%s\"", c->name);
+                list_of_windows = clientListTransientOrModal (c);
+                clientPassFocus (screen_info, c, list_of_windows);
+                clientUnframe (c, FALSE);
+                g_list_free (list_of_windows);
+            }
             return status;
         }
 
@@ -1380,7 +1382,7 @@ handleEnterNotify (DisplayInfo *display_info, XCrossingEvent * ev)
         && screen_info->params->wrap_resistance)
     {
         int msx, msy, maxx, maxy;
-        int rx, ry;
+        int rx, ry, delta;
 
         msx = ev->x_root;
         msy = ev->y_root;
@@ -1436,18 +1438,19 @@ handleEnterNotify (DisplayInfo *display_info, XCrossingEvent * ev)
         if (edge_scroll_x > screen_info->params->wrap_resistance)
         {
             edge_scroll_x = 0;
+            delta = MAX (9 * maxx / 10, maxx - 5 * screen_info->params->wrap_resistance);
             if (msx == 0)
             {
                 if (workspaceMove (screen_info, 0, -1, NULL, ev->time))
                 {
-                    rx = 4 * maxx / 5;
+                    rx = delta;
                 }
             }
             else
             {
                 if (workspaceMove (screen_info, 0, 1, NULL, ev->time))
                 {
-                    rx = -4 * maxx / 5;
+                    rx = -delta;
                 }
             }
             warp_pointer = TRUE;
@@ -1455,18 +1458,19 @@ handleEnterNotify (DisplayInfo *display_info, XCrossingEvent * ev)
         if (edge_scroll_y > screen_info->params->wrap_resistance)
         {
             edge_scroll_y = 0;
+            delta = MAX (9 * maxy / 10, maxy - 5 * screen_info->params->wrap_resistance);
             if (msy == 0)
             {
                 if (workspaceMove (screen_info, -1, 0, NULL, ev->time))
                 {
-                    ry = 4 * maxy / 5;
+                    ry = delta;
                 }
             }
             else
             {
                 if (workspaceMove (screen_info, 1, 0, NULL, ev->time))
                 {
-                    ry = -4 * maxy / 5;
+                    ry = -delta;
                 }
             }
             warp_pointer = TRUE;
@@ -1727,6 +1731,13 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
         else if (ev->atom == XA_WM_HINTS)
         {
             TRACE ("client \"%s\" (0x%lx) has received a XA_WM_HINTS notify", c->name, c->window);
+
+            /* Free previous wmhints if any */
+            if (c->wmhints)
+            {
+                XFree (c->wmhints);
+            }
+
             c->wmhints = XGetWMHints (display_info->dpy, c->window);
             if (c->wmhints)
             {
@@ -1797,11 +1808,7 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
         else if (ev->atom == display_info->atoms[NET_WM_USER_TIME])
         {
             TRACE ("client \"%s\" (0x%lx) has received a NET_WM_USER_TIME notify", c->name, c->window);
-            if (getNetWMUserTime (display_info, c->window, &c->user_time) && (c->user_time != 0))
-            {
-                myDisplaySetLastUserTime (display_info, c->user_time);
-                FLAG_SET (c->flags, CLIENT_FLAG_HAS_USER_TIME);
-            }
+            clientGetUserTime (c);
         }
         else if (ev->atom == display_info->atoms[NET_WM_USER_TIME_WINDOW])
         {
@@ -1876,7 +1883,7 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
     if (ev->atom == display_info->atoms[NET_DESKTOP_NAMES])
     {
         gchar **names;
-        int items;
+        guint items;
 
         TRACE ("root has received a NET_DESKTOP_NAMES notify");
         if (getUTF8StringList (display_info, screen_info->xroot, NET_DESKTOP_NAMES, &names, &items))
@@ -1932,7 +1939,7 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
         else if ((ev->message_type == display_info->atoms[WIN_LAYER]) && (ev->format == 32))
         {
             TRACE ("client \"%s\" (0x%lx) has received a WIN_LAYER event", c->name, c->window);
-            if (ev->data.l[0] != c->win_layer)
+            if ((unsigned long) ev->data.l[0] != c->win_layer)
             {
                 clientSetLayer (c, ev->data.l[0]);
             }
@@ -1940,9 +1947,9 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
         else if ((ev->message_type == display_info->atoms[WIN_WORKSPACE]) && (ev->format == 32))
         {
             TRACE ("client \"%s\" (0x%lx) has received a WIN_WORKSPACE event", c->name, c->window);
-            if (ev->data.l[0] != c->win_workspace)
+            if (ev->data.l[0] != (long) c->win_workspace)
             {
-                clientSetWorkspace (c, ev->data.l[0], TRUE);
+                clientSetWorkspace (c, (guint) ev->data.l[0], TRUE);
             }
         }
         else if ((ev->message_type == display_info->atoms[NET_WM_DESKTOP]) && (ev->format == 32))
@@ -1963,9 +1970,9 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
                     clientUnstick (c, TRUE);
                     frameQueueDraw (c, FALSE);
                 }
-                if (ev->data.l[0] != c->win_workspace)
+                if (ev->data.l[0] != (long) c->win_workspace)
                 {
-                    clientSetWorkspace (c, ev->data.l[0], TRUE);
+                    clientSetWorkspace (c, (guint) ev->data.l[0], TRUE);
                 }
             }
         }
@@ -1991,34 +1998,8 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
         }
         else if ((ev->message_type == display_info->atoms[NET_ACTIVE_WINDOW]) && (ev->format == 32))
         {
-            gboolean source_is_application;
-            guint32 ev_time;
-
-            ev_time = myDisplayGetTime (display_info, (guint32) ev->data.l[1]);
-            source_is_application = (ev->data.l[0] == 1);
-
             TRACE ("client \"%s\" (0x%lx) has received a NET_ACTIVE_WINDOW event", c->name, c->window);
-            if (source_is_application)
-            {
-                guint32 current = myDisplayGetLastUserTime (display_info);
-
-                TRACE ("Time of event received is %u, current XServer time is %u", (guint32) ev_time, (guint32) current);
-                if ((screen_info->params->prevent_focus_stealing) && TIMESTAMP_IS_BEFORE(ev_time, current))
-                {
-                    TRACE ("Setting WM_STATE_DEMANDS_ATTENTION flag on \"%s\" (0x%lx)", c->name, c->window);
-                    FLAG_SET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
-                    clientSetNetState (c);
-                }
-                else
-                {
-                    clientActivate (c, ev_time);
-                }
-            }
-            else
-            {
-                /* The request is either from a pager or an older client, use the most accurate timestamp */
-                clientActivate (c, getXServerTime (display_info));
-            }
+            clientHandleNetActiveWindow (c, (guint32) ev->data.l[1], (gboolean) (ev->data.l[0] == 1));
         }
         else if (ev->message_type == display_info->atoms[NET_REQUEST_FRAME_EXTENTS])
         {
@@ -2046,8 +2027,8 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
              (ev->message_type == display_info->atoms[NET_CURRENT_DESKTOP])) && (ev->format == 32))
         {
             TRACE ("root has received a win_workspace or a NET_CURRENT_DESKTOP event %li", ev->data.l[0]);
-            if ((ev->data.l[0] >= 0) && (ev->data.l[0] < screen_info->workspace_count) &&
-                (ev->data.l[0] != screen_info->current_ws))
+            if ((ev->data.l[0] >= 0) && (ev->data.l[0] < (long) screen_info->workspace_count) &&
+                (ev->data.l[0] != (long) screen_info->current_ws))
             {
                 workspaceSwitch (screen_info, ev->data.l[0], NULL, TRUE,
                                  myDisplayGetTime (display_info, (guint32) ev->data.l[1]));
@@ -2057,7 +2038,7 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
                   (ev->message_type == display_info->atoms[NET_NUMBER_OF_DESKTOPS])) && (ev->format == 32))
         {
             TRACE ("root has received a win_workspace_count event");
-            if (ev->data.l[0] != screen_info->workspace_count)
+            if (ev->data.l[0] != (long) screen_info->workspace_count)
             {
                 workspaceSetCount (screen_info, ev->data.l[0]);
                 getDesktopLayout(display_info, screen_info->xroot, screen_info->workspace_count, &screen_info->desktop_layout);
@@ -2103,7 +2084,7 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
         }
         else if (ev->message_type == display_info->atoms[WM_PROTOCOLS])
         {
-            if (ev->data.l[0] == display_info->atoms[NET_WM_PING])
+            if ((Atom) ev->data.l[0] == display_info->atoms[NET_WM_PING])
             {
                 TRACE ("root has received a NET_WM_PING (pong) event\n");
                 clientReceiveNetWMPong (screen_info, (guint32) ev->data.l[1]);
@@ -2313,6 +2294,7 @@ handleEvent (DisplayInfo *display_info, XEvent * ev)
                 status = handleXSyncAlarmNotify (display_info, (XSyncAlarmNotifyEvent *) ev);
             }
 #endif /* HAVE_XSYNC */
+            break;
     }
     if (!gdk_events_pending () && !XPending (display_info->dpy))
     {
@@ -2327,7 +2309,7 @@ handleEvent (DisplayInfo *display_info, XEvent * ev)
              * Qutting on purpose, update session manager so
              * it does not restart the program immediately
              */
-            client_session_set_restart_style(display_info->session, SESSION_RESTART_IF_RUNNING);
+            xfce_sm_client_set_restart_style(display_info->session, XFCE_SM_CLIENT_RESTART_NORMAL);
             gtk_main_quit ();
         }
     }
@@ -2404,8 +2386,11 @@ menu_callback (Menu * menu, MenuOp op, Window xid, gpointer menu_data, gpointer
                 clientWithdrawAll (c, c->win_workspace);
                 break;
             case MENU_OP_UNMINIMIZE:
+                if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_WAS_SHOWN))
+                {
+                    clientClearAllShowDesktop (c->screen_info);
+                }
                 clientShow (c, TRUE);
-                clientClearAllShowDesktop (c->screen_info);
                 break;
             case MENU_OP_SHADE:
             case MENU_OP_UNSHADE:
@@ -2457,7 +2442,7 @@ initMenuEventWin (void)
 }
 
 static void
-show_window_menu (Client *c, gint px, gint py, guint button, guint32 time)
+show_window_menu (Client *c, gint px, gint py, guint button, guint32 timestamp)
 {
     ScreenInfo *screen_info;
     DisplayInfo *display_info;
@@ -2650,7 +2635,7 @@ show_window_menu (Client *c, gint px, gint py, guint button, guint32 time)
                          screen_info->workspace_names, screen_info->workspace_names_items,
                          display_info->xfilter, screen_info);
 
-    if (!menu_popup (menu, x, y, button, time))
+    if (!menu_popup (menu, x, y, button, timestamp))
     {
         TRACE ("Cannot open menu");
         gdk_beep ();
@@ -2719,25 +2704,63 @@ client_event_cb (GtkWidget * widget, GdkEventClient * ev, gpointer data)
     return (FALSE);
 }
 
+static gboolean
+refresh_font_cb (GObject * obj, GdkEvent * ev, gpointer data)
+{
+    ScreenInfo *screen_info;
+
+    screen_info = (ScreenInfo *) data;
+    g_return_val_if_fail (screen_info, TRUE);
+
+    myScreenUpdateFontHeight (screen_info);
+    clientUpdateAllFrames (screen_info, UPDATE_FRAME);
+    return (TRUE);
+}
+
 static void
 size_changed_cb(GdkScreen *gscreen, gpointer data)
 {
     ScreenInfo *screen_info;
     DisplayInfo *display_info;
+    gboolean size_changed;
 
     TRACE ("entering size_changed_cb");
 
     screen_info = (ScreenInfo *) data;
     g_return_if_fail (screen_info);
-
     display_info = screen_info->display_info;
-    screen_info->width = WidthOfScreen (screen_info->xscreen);
-    screen_info->height = HeightOfScreen (screen_info->xscreen);
+
+    if (gdk_screen_get_n_monitors (screen_info->gscr) == 0)
+    {
+        /*
+         * Recent Xorg drivers disable the output when the lid
+         * is closed, leaving no active monitor, in that case simply
+         * ignore the event to avoid messing with windows' positions.
+         */
+        return;
+    }
+
+    /*
+     * We have added/removed a monitor or even changed the layout,
+     * the cache for monitor position we use in our screen structure
+     * is not valid anymore and potentially refers to a monitor that
+     * was just removed, so invalidate it.
+     */
+    screen_info->cache_monitor.x = -1;
+    screen_info->cache_monitor.y = -1;
+    screen_info->cache_monitor.width = 0;
+    screen_info->cache_monitor.height = 0;
+
+    size_changed = myScreenComputeSize (screen_info);
     setNetWorkarea (display_info, screen_info->xroot, screen_info->workspace_count,
                     screen_info->width, screen_info->height, screen_info->margins);
     placeSidewalks (screen_info, screen_info->params->wrap_workspaces);
     clientScreenResize (screen_info);
-    compositorUpdateScreenSize (screen_info);
+
+    if (size_changed)
+    {
+        compositorUpdateScreenSize (screen_info);
+    }
 }
 
 static void
@@ -2750,16 +2773,6 @@ monitors_changed_cb(GdkScreen *gscreen, gpointer data)
     screen_info = (ScreenInfo *) data;
     g_return_if_fail (screen_info);
 
-    /* We have added/removed a monitor or even changed the layout,
-     * the cache for monitor position we use in our screen structure
-     * is not valid anymore and potentially refers to a monitor that
-     * was just removed, so invalidate it.
-     */
-    screen_info->cache_monitor.x = -1;
-    screen_info->cache_monitor.y = -1;
-    screen_info->cache_monitor.width = 0;
-    screen_info->cache_monitor.height = 0;
-
     /*
      * From the window manager point of view,
      * a XRand 1.2 monitor change is similar to
@@ -2781,7 +2794,7 @@ initGtkCallbacks (ScreenInfo *screen_info)
     g_signal_connect (G_OBJECT(screen_info->gscr), "size-changed",
                       G_CALLBACK(size_changed_cb),
                       (gpointer) (screen_info));
-    if(gtk_major_version > 2 || (gtk_major_version == 2 && gtk_minor_version >= 13))
+    if (gtk_major_version > 2 || (gtk_major_version == 2 && gtk_minor_version >= 13))
     {
         TRACE ("connect \"monitors-changed\" cb");
         g_signal_connect (G_OBJECT(screen_info->gscr), "monitors-changed",
@@ -2798,6 +2811,16 @@ initGtkCallbacks (ScreenInfo *screen_info)
             G_CALLBACK (set_reload), (gpointer) (screen_info->display_info));
         g_signal_connect (settings, "notify::gtk-double-click-time",
             G_CALLBACK (dbl_click_time_cb), (gpointer) (screen_info->display_info));
+        g_signal_connect_after (settings, "notify::gtk-xft-antialias",
+            G_CALLBACK (refresh_font_cb), (gpointer) (screen_info));
+        g_signal_connect_after (settings, "notify::gtk-xft-dpi",
+            G_CALLBACK (refresh_font_cb), (gpointer) (screen_info));
+        g_signal_connect_after (settings, "notify::gtk-xft-hinting",
+            G_CALLBACK (refresh_font_cb), (gpointer) (screen_info));
+        g_signal_connect_after (settings, "notify::gtk-xft-hintstyle",
+            G_CALLBACK (refresh_font_cb), (gpointer) (screen_info));
+        g_signal_connect_after (settings, "notify::gtk-xft-rgba",
+            G_CALLBACK (refresh_font_cb), (gpointer) (screen_info));
     }
 }
 
