@@ -1,4 +1,4 @@
-/*      $Id: menu.c 28506 2008-10-29 23:10:25Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 
 
         Metacity - (c) 2001 Havoc Pennington
-        xfwm4    - (c) 2002-2008 Olivier Fourdan
+        xfwm4    - (c) 2002-2009 Olivier Fourdan
 
  */
 
@@ -126,7 +126,7 @@ popup_position_func (GtkMenu * menu, gint * x, gint * y, gboolean * push_in,
     }
 
     screen = gtk_widget_get_screen (GTK_WIDGET(menu));
-    monitor = find_monitor_at_point (screen, *x, *y);
+    monitor = gdk_screen_get_monitor_at_point (screen, *x, *y);
     gtk_menu_set_monitor (GTK_MENU (menu), monitor);
 
     g_free (user_data);
@@ -251,6 +251,7 @@ menu_default (GdkScreen *gscr, Window xid, MenuOp ops, MenuOp insensitive, MenuF
                     }
                     ws_menu = menu_workspace (menu, insensitive, ws, nws, wsn, wsn_items);
                     gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), ws_menu);
+                    g_signal_connect (GTK_OBJECT (ws_menu), "selection-done", GTK_SIGNAL_FUNC (menu_closed), menu);
                     break;
                 default:
                     if (menuitems[i].image_name)
@@ -282,22 +283,11 @@ menu_default (GdkScreen *gscr, Window xid, MenuOp ops, MenuOp insensitive, MenuF
         }
         ++i;
     }
-    menu_connect (menu);
+    g_signal_connect (GTK_OBJECT (menu->menu), "selection-done", GTK_SIGNAL_FUNC (menu_closed), menu);
 
     return (menu);
 }
 
-Menu *
-menu_connect (Menu * menu)
-{
-    TRACE ("entering menu_connect");
-    g_return_val_if_fail (menu != NULL, NULL);
-    g_return_val_if_fail (GTK_IS_MENU (menu->menu), NULL);
-    g_signal_connect (GTK_OBJECT (menu->menu), "selection_done",
-        GTK_SIGNAL_FUNC (menu_closed), menu);
-    return (menu);
-}
-
 static void
 closure_notify (gpointer data, GClosure * closure)
 {
