@@ -35,10 +35,11 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#
+
 #include "client.h"
 #include "focus.h"
 #include "frame.h"
+#include "moveresize.h"
 #include "placement.h"
 #include "poswin.h"
 #include "screen.h"
@@ -63,15 +64,15 @@ struct _MoveResizeData
     gboolean is_transient;
     gboolean move_resized;
     gboolean released;
-    int button;
-    int cancel_x, cancel_y;
-    int cancel_w, cancel_h;
-    int cancel_workspace;
-    int mx, my;
-    int ox, oy;
-    int ow, oh;
-    int oldw, oldh;
-    int handle;
+    guint button;
+    gint cancel_x, cancel_y;
+    gint cancel_w, cancel_h;
+    guint cancel_workspace;
+    gint mx, my;
+    gint ox, oy;
+    gint ow, oh;
+    gint oldw, oldh;
+    gint handle;
     Poswin *poswin;
 };
 
@@ -187,7 +188,7 @@ clientSetHeight (Client * c, int h)
 static void
 clientMovePointer (DisplayInfo *display_info, gint dx, gint dy, guint repeat)
 {
-    gint i;
+    guint i;
     for (i = 0; i < repeat; ++i)
     {
         XWarpPointer (display_info->dpy, None, None, 0, 0, 0, 0, dx, dy);
@@ -271,7 +272,7 @@ clientSetHandle(MoveResizeData *passdata, int handle)
  */
 
 #define MAKE_MULT(a,b) ((b==1) ? (a) : (((int)((a)/(b))) * (b)) )
-void
+static void
 clientConstrainRatio (Client * c, int *w, int *h, int handle)
 {
 
@@ -364,12 +365,121 @@ clientDrawOutline (Client * c)
     }
 }
 
+static gboolean
+clientCheckOverlap (int s1, int e1, int s2, int e2)
+{
+    /* Simple overlap test for an arbitary axis. -Cliff */
+    if ((s1 >= s2 && s1 <= e2) ||
+        (e1 >= s2 && e1 <= e2) ||
+        (s2 >= s1 && s2 <= e1) ||
+        (e2 >= s1 && e2 <= e1))
+    {
+        return TRUE;
+    }
+    return FALSE;
+}
+
+static int
+clientFindClosestEdgeX (Client *c, int edge_pos)
+{
+    /* Find the closest edge of anything that we can snap to, taking
+       frames into account, or just return the original value if nothing
+       is within the snapping range. -Cliff */
+
+    Client *c2;
+    ScreenInfo *screen_info;
+    guint i;
+    int snap_width, closest;
+
+    screen_info = c->screen_info;
+    snap_width = screen_info->params->snap_width;
+    closest = edge_pos + snap_width + 2; /* This only needs to be out of the snap range to work. -Cliff */
+
+    for (c2 = screen_info->clients, i = 0; i < screen_info->client_count; c2 = c2->next, i++)
+    {
+        if (FLAG_TEST (c2->xfwm_flags, XFWM_FLAG_VISIBLE)  && (c2 != c) &&
+            (((screen_info->params->snap_to_windows) && (c2->win_layer == c->win_layer))
+             || ((screen_info->params->snap_to_border)
+                  && FLAG_TEST (c2->flags, CLIENT_FLAG_HAS_STRUT)
+                  && FLAG_TEST (c2->xfwm_flags, XFWM_FLAG_VISIBLE))))
+        {
+
+            if (clientCheckOverlap (c->y - frameTop (c) - 1, c->y + c->height + frameBottom (c) + 1, c2->y - frameTop (c) - 1, c2->y + c2->height + frameBottom (c) + 1))
+            {
+                if (abs (c2->x - frameLeft (c2) - edge_pos) < abs (closest - edge_pos))
+                {
+                    closest = c2->x - frameLeft (c2);
+                }
+                if (abs ((c2->x + c2->width) + frameRight (c2) - edge_pos) < abs (closest - edge_pos))
+                {
+                    closest = (c2->x + c2->width) + frameRight (c2);
+                }
+            }
+        }
+    }
+
+    if (abs (closest - edge_pos) > snap_width)
+    {
+        closest = edge_pos;
+    }
+
+    return closest;
+}
+
+static int
+clientFindClosestEdgeY (Client *c, int edge_pos)
+{
+    /* This function is mostly identical to the one above, but swaps the
+       axes. If there's a better way to do it than this, I'd like to
+       know. -Cliff */
+
+    Client *c2;
+    ScreenInfo *screen_info;
+    guint i;
+    int snap_width, closest;
+
+    screen_info = c->screen_info;
+    snap_width = screen_info->params->snap_width;
+    closest = edge_pos + snap_width + 1; /* This only needs to be out of the snap range to work. -Cliff */
+
+    for (c2 = screen_info->clients, i = 0; i < screen_info->client_count; c2 = c2->next, i++)
+    {
+        if (FLAG_TEST (c2->xfwm_flags, XFWM_FLAG_VISIBLE)  && (c2 != c) &&
+            (((screen_info->params->snap_to_windows) && (c2->win_layer == c->win_layer))
+             || ((screen_info->params->snap_to_border)
+                  && FLAG_TEST (c2->flags, CLIENT_FLAG_HAS_STRUT)
+                  && FLAG_TEST (c2->xfwm_flags, XFWM_FLAG_VISIBLE))))
+        {
+
+            if (clientCheckOverlap (c->x - frameLeft (c) - 1, c->x + c->width + frameRight (c) + 1, c2->x - frameLeft (c) - 1, c2->x + c2->width + frameRight (c) + 1))
+            {
+                if (abs (c2->y - frameTop(c2) - edge_pos) < abs (closest - edge_pos))
+                {
+                    closest = c2->y - frameTop (c2);
+                }
+                if (abs ((c2->y + c2->height) + frameBottom (c2) - edge_pos) < abs (closest - edge_pos))
+                {
+                    closest = (c2->y + c2->height) + frameBottom (c2);
+                }
+            }
+        }
+    }
+
+    if (abs (closest - edge_pos) > snap_width)
+    {
+        closest = edge_pos;
+    }
+
+    return closest;
+}
+
 static void
 clientSnapPosition (Client * c, int prev_x, int prev_y)
 {
     ScreenInfo *screen_info;
     Client *c2;
-    int cx, cy, i, delta;
+    guint i;
+    int cx, cy, delta;
     int disp_x, disp_y, disp_max_x, disp_max_y;
     int frame_x, frame_y, frame_height, frame_width;
     int frame_top, frame_left, frame_right, frame_bottom;
@@ -552,7 +662,7 @@ clientMoveEventFilter (XEvent * xevent, gpointer data)
     gboolean moving = TRUE;
     gboolean warp_pointer = FALSE;
     XWindowChanges wc;
-    int prev_x, prev_y;
+    int prev_x, prev_y, delta;
 
     TRACE ("entering clientMoveEventFilter");
 
@@ -722,18 +832,19 @@ clientMoveEventFilter (XEvent * xevent, gpointer data)
                     edge_scroll_x = 0;
                     if ((msx == 0) || (msx == maxx))
                     {
+                        delta = MAX (9 * maxx / 10, maxx - 5 * screen_info->params->wrap_resistance);
                         if (msx == 0)
                         {
                             if (workspaceMove (screen_info, 0, -1, c, xevent->xmotion.time))
                             {
-                                rx = 4 * maxx / 5;
+                                rx = delta;
                             }
                         }
                         else
                         {
                             if (workspaceMove (screen_info, 0, 1, c, xevent->xmotion.time))
                             {
-                                rx = -4 * maxx / 5;
+                                rx = -delta;
                             }
                         }
                         warp_pointer = TRUE;
@@ -745,18 +856,19 @@ clientMoveEventFilter (XEvent * xevent, gpointer data)
                     edge_scroll_y = 0;
                     if ((msy == 0) || (msy == maxy))
                     {
+                        delta = MAX (9 * maxy / 10, maxy - 5 * screen_info->params->wrap_resistance);
                         if (msy == 0)
                         {
                             if (workspaceMove (screen_info, -1, 0, c, xevent->xmotion.time))
                             {
-                                ry = 4 * maxy / 5;
+                                ry = delta;
                             }
                         }
                         else
                         {
                             if (workspaceMove (screen_info, 1, 0, c, xevent->xmotion.time))
                             {
-                                ry = -4 * maxy / 5;
+                                ry = -delta;
                             }
                         }
                         warp_pointer = TRUE;
@@ -1020,6 +1132,7 @@ clientMove (Client * c, XEvent * ev)
         gtk_main ();
         eventFilterPop (display_info->xfilter);
     }
+
     myScreenUngrabKeyboard (screen_info, myDisplayGetCurrentTime (display_info));
     myScreenUngrabPointer (screen_info, myDisplayGetCurrentTime (display_info));
 
@@ -1106,6 +1219,8 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
     int temp;
     gint min_visible;
     gboolean resizing;
+    int right_edge; /* -Cliff */
+    int bottom_edge; /* -Cliff */
 
     TRACE ("entering clientResizeEventFilter");
 
@@ -1293,6 +1408,10 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
         else if (move_right)
         {
             c->width = passdata->ow + (xevent->xmotion.x_root - passdata->mx);
+
+            /* Attempt to snap the right edge to something. -Cliff */
+            c->width = clientFindClosestEdgeX (c, c->x + c->width + frameRight (c)) - c->x - frameRight (c);
+
         }
         if (!FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
         {
@@ -1303,6 +1422,9 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
             else if (move_bottom)
             {
                 c->height = passdata->oh + (xevent->xmotion.y_root - passdata->my);
+
+                /* Attempt to snap the bottom edge to something. -Cliff */
+                c->height = clientFindClosestEdgeY (c, c->y + c->height + frameBottom (c)) - c->y - frameBottom (c);
             }
         }
         clientConstrainRatio (c, &c->width, &c->height, passdata->handle);
@@ -1311,6 +1433,12 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
         if (move_left)
         {
             c->x = c->x - (c->width - passdata->oldw);
+
+            /* Snap the left edge to something. -Cliff */
+            right_edge = c->x + c->width;
+            c->x = clientFindClosestEdgeX (c, c->x - frameLeft (c)) + frameLeft (c);
+            c->width = right_edge - c->x;
+
             frame_x = frameX (c);
         }
 
@@ -1318,6 +1446,12 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
         if (!FLAG_TEST (c->flags, CLIENT_FLAG_SHADED) && move_top)
         {
             c->y = c->y - (c->height - passdata->oldh);
+
+            /* Snap the top edge to something. -Cliff */
+            bottom_edge = c->y + c->height;
+            c->y = clientFindClosestEdgeY (c, c->y - frameTop (c)) + frameTop (c);
+            c->height = bottom_edge - c->y;
+
             frame_y = frameY (c);
         }
 
@@ -1561,6 +1695,7 @@ clientResize (Client * c, int handle, XEvent * ev)
         gtk_main ();
         eventFilterPop (display_info->xfilter);
     }
+
     myScreenUngrabKeyboard (screen_info, myDisplayGetCurrentTime (display_info));
     myScreenUngrabPointer (screen_info, myDisplayGetCurrentTime (display_info));
 
