@@ -367,7 +367,7 @@ xpm_extract_color (const gchar *buffer, xfwmColorSymbol *color_sym)
         for (r = word;
                  (*p != '\0') &&
                  (!g_ascii_isspace (*p)) &&
-                 (r - word < sizeof (word) - 1);
+                 (r - word < (gint) sizeof (word) - 1);
              p++, r++)
         {
                 *r = *p;
@@ -428,7 +428,7 @@ xpm_extract_color (const gchar *buffer, xfwmColorSymbol *color_sym)
                 space -= MIN (space, 1);
             }
             strncat (color, word, space);
-            space -= MIN (space, strlen (word));
+            space -= MIN (space, (gint) strlen (word));
         }
         else if (key == 5)
         {
@@ -547,7 +547,7 @@ pixbuf_create_from_xpm (gpointer handle, xfwmColorSymbol *color_sym)
         (cpp >= 32) ||
         (n_col <= 0) ||
         (n_col >= G_MAXINT / (cpp + 1)) ||
-        (n_col >= G_MAXINT / sizeof (XPMColor)))
+        (n_col >= G_MAXINT / (gint) sizeof (XPMColor)))
     {
         g_warning ("Pixmap definition contains invalid attributes");
         return NULL;
@@ -632,7 +632,7 @@ pixbuf_create_from_xpm (gpointer handle, xfwmColorSymbol *color_sym)
         pixtmp = gdk_pixbuf_get_pixels (pixbuf) + ycnt * gdk_pixbuf_get_rowstride(pixbuf);
 
         buffer = file_buffer (op_body, handle);
-        if ((!buffer) || (strlen (buffer) < wbytes))
+        if ((!buffer) || (wbytes > (gint) strlen (buffer)))
         {
             continue;
         }
@@ -705,10 +705,10 @@ xpm_image_load (const char *filename, xfwmColorSymbol *color_sym)
     return pixbuf;
 }
 
+#ifdef HAVE_RENDER
 static void
 xfwmPixmapRefreshPict (xfwmPixmap * pm)
 {
-#ifdef HAVE_RENDER
     ScreenInfo * screen_info;
 
     screen_info = pm->screen_info;
@@ -729,8 +729,8 @@ xfwmPixmapRefreshPict (xfwmPixmap * pm)
         pm->pict = XRenderCreatePicture (myScreenGetXDisplay (screen_info),
                                          pm->pixmap, pm->pict_format, 0, NULL);
     }
-#endif
 }
+#endif
 
 static GdkPixbuf *
 xfwmPixmapCompose (GdkPixbuf *pixbuf, const gchar * dir, const gchar * file)
