@@ -1,4 +1,4 @@
-/*      $Id: screen.c 27421 2008-08-04 20:28:44Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
         MA 02110-1301, USA.
 
 
-        xfwm4    - (c) 2002-2008 Olivier Fourdan
+        xfwm4    - (c) 2002-2009 Olivier Fourdan
 
  */
 
@@ -31,6 +31,7 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
+#include <pango/pango.h>
 #include <libxfce4util/libxfce4util.h>
 
 #ifdef HAVE_RENDER
@@ -47,6 +48,7 @@
 #include "misc.h"
 #include "mywindow.h"
 #include "compositor.h"
+#include "ui_style.h"
 
 gboolean
 myScreenCheckWMAtom (ScreenInfo *screen_info, Atom atom)
@@ -206,7 +208,7 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
     screen_info->margins[STRUTS_RIGHT] = screen_info->gnome_margins[STRUTS_RIGHT] = 0;
     screen_info->margins[STRUTS_BOTTOM] = screen_info->gnome_margins[STRUTS_BOTTOM] = 0;
 
-    screen_info->workspace_count = -1;
+    screen_info->workspace_count = 0;
     screen_info->workspace_names = NULL;
     screen_info->workspace_names_items = 0;
 
@@ -310,6 +312,8 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
         xfwmPixmapInit (screen_info, &screen_info->top[i][INACTIVE]);
     }
 
+    myScreenInvalidateMonitorCache (screen_info);
+
     return (screen_info);
 }
 
@@ -387,7 +391,7 @@ myScreenGetGdkWindow (ScreenInfo *screen_info)
 }
 
 gboolean
-myScreenGrabKeyboard (ScreenInfo *screen_info, Time time)
+myScreenGrabKeyboard (ScreenInfo *screen_info, guint32 timestamp)
 {
     gboolean grab;
 
@@ -399,10 +403,10 @@ myScreenGrabKeyboard (ScreenInfo *screen_info, Time time)
     if (screen_info->key_grabs == 0)
     {
         grab = (XGrabKeyboard (myScreenGetXDisplay (screen_info),
-                               screen_info->xfwm4_win,
-                               FALSE,
+                               screen_info->xroot,
+                               TRUE,
                                GrabModeAsync, GrabModeAsync,
-                               time) == GrabSuccess);
+                               (Time) timestamp) == GrabSuccess);
     }
     screen_info->key_grabs++;
     TRACE ("global key grabs %i", screen_info->key_grabs);
@@ -411,7 +415,7 @@ myScreenGrabKeyboard (ScreenInfo *screen_info, Time time)
 }
 
 gboolean
-myScreenGrabPointer (ScreenInfo *screen_info, unsigned int event_mask, Cursor cursor, Time time)
+myScreenGrabPointer (ScreenInfo *screen_info, unsigned int event_mask, Cursor cursor, guint32 timestamp)
 {
     gboolean grab;
 
@@ -422,12 +426,12 @@ myScreenGrabPointer (ScreenInfo *screen_info, unsigned int event_mask, Cursor cu
     if (screen_info->pointer_grabs == 0)
     {
         grab = (XGrabPointer (myScreenGetXDisplay (screen_info),
-                              screen_info->xfwm4_win,
+                              screen_info->xroot,
                               FALSE, event_mask,
                               GrabModeAsync, GrabModeAsync,
                               screen_info->xroot,
                               cursor,
-                              time) == GrabSuccess);
+                              (Time) timestamp) == GrabSuccess);
     }
     screen_info->pointer_grabs++;
     TRACE ("global pointer grabs %i", screen_info->pointer_grabs);
@@ -436,7 +440,7 @@ myScreenGrabPointer (ScreenInfo *screen_info, unsigned int event_mask, Cursor cu
 }
 
 gboolean
-myScreenChangeGrabPointer (ScreenInfo *screen_info, unsigned int event_mask, Cursor cursor, Time time)
+myScreenChangeGrabPointer (ScreenInfo *screen_info, unsigned int event_mask, Cursor cursor, guint32 timestamp)
 {
     gboolean grab;
 
@@ -447,14 +451,14 @@ myScreenChangeGrabPointer (ScreenInfo *screen_info, unsigned int event_mask, Cur
     if (screen_info->pointer_grabs > 0)
     {
         grab = (XChangeActivePointerGrab (myScreenGetXDisplay (screen_info),
-                                          event_mask, cursor, time) == GrabSuccess);
+                                          event_mask, cursor, (Time) timestamp) == GrabSuccess);
     }
 
     return grab;
 }
 
 unsigned int
-myScreenUngrabKeyboard (ScreenInfo *screen_info, Time time)
+myScreenUngrabKeyboard (ScreenInfo *screen_info, guint32 timestamp)
 {
     g_return_val_if_fail (screen_info, 0);
     TRACE ("entering myScreenUngrabKeyboard");
@@ -466,7 +470,7 @@ myScreenUngrabKeyboard (ScreenInfo *screen_info, Time time)
     }
     if (screen_info->key_grabs == 0)
     {
-        XUngrabKeyboard (myScreenGetXDisplay (screen_info), time);
+        XUngrabKeyboard (myScreenGetXDisplay (screen_info), (Time) timestamp);
     }
     TRACE ("global key grabs %i", screen_info->key_grabs);
 
@@ -474,7 +478,7 @@ myScreenUngrabKeyboard (ScreenInfo *screen_info, Time time)
 }
 
 unsigned int
-myScreenUngrabPointer (ScreenInfo *screen_info, Time time)
+myScreenUngrabPointer (ScreenInfo *screen_info, guint32 timestamp)
 {
     g_return_val_if_fail (screen_info, 0);
     TRACE ("entering myScreenUngrabPointer");
@@ -486,7 +490,7 @@ myScreenUngrabPointer (ScreenInfo *screen_info, Time time)
     }
     if (screen_info->pointer_grabs == 0)
     {
-        XUngrabPointer (myScreenGetXDisplay (screen_info), time);
+        XUngrabPointer (myScreenGetXDisplay (screen_info), (Time) timestamp);
     }
     TRACE ("global pointer grabs %i", screen_info->pointer_grabs);
 
@@ -522,11 +526,31 @@ myScreenUngrabKeys (ScreenInfo *screen_info)
     ungrabKeys (dpy, screen_info->xroot);
 }
 
+int
+myScreenGetKeyPressed (ScreenInfo *screen_info, XKeyEvent * ev)
+{
+    int key, state;
+
+    TRACE ("entering myScreenGetKeyPressed");
+
+    state = ev->state & MODIFIER_MASK;
+    for (key = 0; key < KEY_COUNT; key++)
+    {
+        if ((screen_info->params->keys[key].keycode == ev->keycode)
+            && (screen_info->params->keys[key].modifier == state))
+        {
+            break;
+        }
+    }
+
+    return key;
+}
+
 Client *
 myScreenGetClientFromWindow (ScreenInfo *screen_info, Window w, unsigned short mode)
 {
     Client *c;
-    int i;
+    guint i;
 
     g_return_val_if_fail (w != None, NULL);
     TRACE ("entering myScreenGetClientFromWindow");
@@ -543,3 +567,140 @@ myScreenGetClientFromWindow (ScreenInfo *screen_info, Window w, unsigned short m
 
     return NULL;
 }
+
+gboolean
+myScreenComputeSize (ScreenInfo *screen_info)
+{
+    gint num_monitors, i;
+    gint width, height;
+    GdkRectangle monitor;
+    gboolean changed;
+
+    g_return_val_if_fail (screen_info != NULL, FALSE);
+    g_return_val_if_fail (GDK_IS_SCREEN (screen_info->gscr), FALSE);
+    TRACE ("entering myScreenComputeSize");
+
+    width = 0;
+    height = 0;
+    num_monitors = gdk_screen_get_n_monitors (screen_info->gscr);
+
+    if (num_monitors == 0)
+    {
+        return FALSE;
+    }
+
+    for (i = 0; i < num_monitors; i++)
+    {
+        gdk_screen_get_monitor_geometry (screen_info->gscr, i, &monitor);
+        width = MAX (monitor.x + monitor.width, width);
+        height = MAX (monitor.y + monitor.height, height);
+    }
+
+    changed = ((screen_info->width != width) | (screen_info->height != height));
+    screen_info->width = width;
+    screen_info->height = height;
+    TRACE ("myScreenComputeSize(): width=%i, height=%i", width, height);
+
+    return changed;
+}
+
+void
+myScreenInvalidateMonitorCache (ScreenInfo *screen_info)
+{
+    g_return_if_fail (screen_info != NULL);
+    TRACE ("entering myScreenInvalidateMonitorCache");
+
+    screen_info->cache_monitor.x = G_MAXINT;
+    screen_info->cache_monitor.y = G_MAXINT;
+    screen_info->cache_monitor.width = 0;
+    screen_info->cache_monitor.height = 0;
+}
+
+/*
+   gdk_screen_get_monitor_at_point () doesn't give accurate results
+   when the point is off screen, use my own implementation from xfce 3
+ */
+void
+myScreenFindMonitorAtPoint (ScreenInfo *screen_info, gint x, gint y, GdkRectangle *rect)
+{
+    gint dx, dy, center_x, center_y, num_monitors, i;
+    guint32 distsquare, min_distsquare;
+    GdkRectangle monitor, nearest_monitor = { G_MAXINT, G_MAXINT, 0, 0 };
+
+    g_return_if_fail (screen_info != NULL);
+    g_return_if_fail (rect != NULL);
+    g_return_if_fail (GDK_IS_SCREEN (screen_info->gscr));
+    TRACE ("entering myScreenFindMonitorAtPoint");
+
+    /* Cache system */
+    if ((x >= screen_info->cache_monitor.x) && (x < screen_info->cache_monitor.x + screen_info->cache_monitor.width) &&
+        (y >= screen_info->cache_monitor.y) && (y < screen_info->cache_monitor.y + screen_info->cache_monitor.height))
+    {
+        *rect = screen_info->cache_monitor;
+        return;
+    }
+
+    min_distsquare = G_MAXUINT32;
+    num_monitors = gdk_screen_get_n_monitors (screen_info->gscr);
+
+    for (i = 0; i < num_monitors; i++)
+    {
+        gdk_screen_get_monitor_geometry (screen_info->gscr, i, &monitor);
+
+        if ((x >= monitor.x) && (x < (monitor.x + monitor.width)) &&
+            (y >= monitor.y) && (y < (monitor.y + monitor.height)))
+        {
+            screen_info->cache_monitor = monitor;
+            *rect = screen_info->cache_monitor;
+            return;
+        }
+
+        center_x = monitor.x + (monitor.width / 2);
+        center_y = monitor.y + (monitor.height / 2);
+
+        dx = x - center_x;
+        dy = y - center_y;
+
+        distsquare = (dx * dx) + (dy * dy);
+
+        if (distsquare < min_distsquare)
+        {
+            min_distsquare = distsquare;
+            nearest_monitor = monitor;
+        }
+    }
+
+    screen_info->cache_monitor = nearest_monitor;
+    *rect = screen_info->cache_monitor;
+}
+
+gboolean
+myScreenUpdateFontHeight (ScreenInfo *screen_info)
+{
+    PangoFontDescription *desc;
+    PangoContext *context;
+    PangoFontMetrics *metrics;
+    PangoLanguage *language;
+    GtkWidget *widget;
+
+    g_return_val_if_fail (screen_info != NULL, FALSE);
+    TRACE ("entering myScreenUpdateFontHeight");
+
+    widget = myScreenGetGtkWidget (screen_info);
+    context = getUIPangoContext (widget);
+    desc = getUIPangoFontDesc (widget);
+
+    if (desc && context)
+    {
+        language = pango_context_get_language (context);
+        metrics = pango_context_get_metrics (context, desc, language);
+        screen_info->font_height =
+                 PANGO_PIXELS (pango_font_metrics_get_ascent (metrics) +
+                               pango_font_metrics_get_descent (metrics));
+        pango_font_metrics_unref (metrics);
+
+        return TRUE;
+    }
+
+    return FALSE;
+}
