@@ -1,4 +1,4 @@
-/*      $Id: session.c 28284 2008-10-17 13:10:03Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
         MA 02110-1301, USA.
 
 
-        xfwm4    - (c) 2002-2008 Olivier Fourdan
+        xfwm4    - (c) 2002-2009 Olivier Fourdan
  */
 
 /* Initially inspired by xfwm, fvwm2, enlightment and twm implementations */
@@ -40,7 +40,7 @@
 
 #include <gtk/gtk.h>
 #include <glib.h>
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4ui/libxfce4ui.h>
 
 #include "display.h"
 #include "screen.h"
@@ -268,10 +268,10 @@ sessionSaveScreen (ScreenInfo *screen_info, FILE *f)
 {
     DisplayInfo *display_info;
     Client *c;
-    char *client_id, *window_role;
-    char **wm_command;
-    int wm_command_count;
-    gint client_idx;
+    gchar *client_id, *window_role;
+    gchar **wm_command;
+    gint wm_command_count;
+    guint client_idx;
 
     display_info = screen_info->display_info;
     wm_command_count = 0;
@@ -366,7 +366,7 @@ sessionSaveScreen (ScreenInfo *screen_info, FILE *f)
 }
 
 gboolean
-sessionSaveWindowStates (DisplayInfo *display_info, gchar * filename)
+sessionSaveWindowStates (DisplayInfo *display_info, const gchar * filename)
 {
     FILE *f;
     GSList *screens;
@@ -388,11 +388,11 @@ sessionSaveWindowStates (DisplayInfo *display_info, gchar * filename)
 }
 
 gboolean
-sessionLoadWindowStates (gchar * filename)
+sessionLoadWindowStates (const gchar * filename)
 {
     FILE *f;
-    char s[4096], s1[4096];
-    int i, pos, pos1;
+    gchar s[4096], s1[4096];
+    gint i, pos, pos1;
     unsigned long w;
 
     g_return_val_if_fail (filename != NULL, FALSE);
@@ -506,7 +506,7 @@ sessionLoadWindowStates (gchar * filename)
 void
 sessionFreeWindowStates (void)
 {
-    int i;
+    gint i;
 
     for (i = 0; i < num_match; i++)
     {
@@ -561,11 +561,11 @@ matchWin (Client * c, Match * m)
 {
     ScreenInfo *screen_info;
     DisplayInfo *display_info;
-    char *client_id;
-    char *window_role;
-    char **wm_command;
-    int wm_command_count;
-    int i;
+    gchar *client_id;
+    gchar *window_role;
+    gchar **wm_command;
+    gint wm_command_count;
+    gint i;
     gboolean found;
 
     g_return_val_if_fail (c != NULL, FALSE);
@@ -677,7 +677,7 @@ matchWin (Client * c, Match * m)
 gboolean
 sessionMatchWinToSM (Client * c)
 {
-    int i;
+    gint i;
 
     g_return_val_if_fail (c != NULL, FALSE);
     for (i = 0; i < num_match; i++)
@@ -705,91 +705,85 @@ sessionMatchWinToSM (Client * c)
     return FALSE;
 }
 
-static char *
-sessionBuildFilename(SessionClient *client_session)
-{
-    gchar *filename, *path, *file;
-    GError *error;
-
-    path = xfce_resource_save_location (XFCE_RESOURCE_CACHE, "sessions", FALSE);
-
-    error = NULL;
-    if (!xfce_mkdirhier(path, 0700, &error))
-    {
-        g_warning("Unable to create session dir %s: %s", path, error->message);
-        g_error_free (error);
-        g_free (path);
-        return NULL;
-    }
-
-    file = g_strdup_printf("xfwm4-%s", client_session->given_client_id);
-    filename = g_build_filename (path, file, NULL);
-    g_free (file);
-    g_free (path);
-
-    return filename;
-}
-
 static void
 sessionLoad (DisplayInfo *display_info)
 {
-    SessionClient *session;
-    gchar *filename;
+    const gchar *filename;
 
-    session = display_info->session;
-    filename = sessionBuildFilename(session);
+    filename = xfce_sm_client_get_state_file (display_info->session);
+    DBG ("Restoring session from \"%s\"", filename);
     if (filename)
     {
         sessionLoadWindowStates (filename);
-        g_free (filename);
     }
 }
 
 static void
-sessionSavePhase2 (gpointer data)
+sessionSavePhase2 (XfceSMClient *session,
+                   DisplayInfo *display_info)
 {
-    DisplayInfo *display_info;
-    SessionClient *session;
-    gchar *filename;
+    const gchar *filename;
 
-    display_info = (DisplayInfo *) data;
-    session = display_info->session;
-    filename = sessionBuildFilename(session);
+    g_return_if_fail (XFCE_IS_SM_CLIENT (session));
+    g_return_if_fail (session == display_info->session);
+
+    filename = xfce_sm_client_get_state_file (display_info->session);
+    DBG ("Saving session to \"%s\"", filename);
     if (filename)
     {
         sessionSaveWindowStates (display_info, filename);
-        g_free (filename);
     }
 }
 
 static void
-sessionDie (gpointer data)
+sessionDie (XfceSMClient *session,
+            DisplayInfo *display_info)
 {
-    DisplayInfo *display_info;
+    g_return_if_fail (XFCE_IS_SM_CLIENT (session));
+    g_return_if_fail (session == display_info->session);
 
-    display_info = (DisplayInfo *) data;
-    client_session_set_restart_style(display_info->session, SESSION_RESTART_IF_RUNNING);    
-    display_info->quit = TRUE;
+    /*
+     * Do not change the session restart style to NORMAL here, else
+     * xfwm4 will never be restarted the next time we login. just
+     * gracefully quit the application.
+     */
+    DBG ("Session clients asked to quit");
     gtk_main_quit ();
 }
 
 int
-sessionStart (int argc, char **argv, DisplayInfo *display_info)
+sessionStart (DisplayInfo *display_info)
 {
-    SessionClient *session;
+    XfceSMClient *session;
+    GError *error = NULL;
 
-    display_info->session = client_session_new (argc, argv, (gpointer) display_info,
-                                                SESSION_RESTART_IMMEDIATELY, 20);
-    session = display_info->session;
-    session->data = (gpointer) display_info;
-    session->save_phase_2 = sessionSavePhase2;
-    session->die = sessionDie;
+    DBG ("Starting session client");
 
-    if (session_init (session))
+    session = xfce_sm_client_get ();
+    xfce_sm_client_set_restart_style (session, XFCE_SM_CLIENT_RESTART_IMMEDIATELY);
+    xfce_sm_client_set_priority (session, XFCE_SM_CLIENT_PRIORITY_WM);
+
+    if (xfce_sm_client_connect(session, &error))
     {
-        sessionLoad (display_info);
+        display_info->session = session;
+
+        if (xfce_sm_client_is_resumed (session))
+            sessionLoad (display_info);
+
+        /* save-state-extended is special for window managers to store
+         * the window positions of all the clients */
+        g_signal_connect (G_OBJECT (session), "save-state-extended",
+                          G_CALLBACK (sessionSavePhase2), display_info);
+        g_signal_connect (G_OBJECT (session), "quit",
+                          G_CALLBACK (sessionDie), display_info);
+
         return 1;
     }
+    else
+    {
+        g_warning ("Failed to connect to session manager: %s", error->message);
+        g_error_free (error);
+    }
 
     return 0;
 }
