@@ -1,4 +1,4 @@
-/*      $Id: session.h 26999 2008-05-31 14:35:42Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
         MA 02110-1301, USA.
 
 
-        xfwm4    - (c) 2002-2008 Olivier Fourdan
+        xfwm4    - (c) 2002-2009 Olivier Fourdan
  */
 
 
@@ -36,12 +36,12 @@
  *  Save window states to file which name is given in argument.
  */
 gboolean                sessionSaveWindowStates                 (DisplayInfo *,
-                                                                 gchar *);
+                                                                 const gchar *);
 
 /*
  *  Load window states to file which name is given in argument.
  */
-gboolean                sessionLoadWindowStates                 (gchar *);
+gboolean                sessionLoadWindowStates                 (const gchar *);
 
 /*
  * Free allocated structure. Should be called before xfwm4 dies
@@ -58,8 +58,6 @@ gboolean                sessionMatchWinToSM                     (Client *);
  * Initiate session, connect to session manager and
  * load saved states if the connection succeeds.
  */
-int                     sessionStart                            (int,
-                                                                 char **,
-                                                                 DisplayInfo *);
+int                     sessionStart                            (DisplayInfo *);
 
 #endif /* INC_CLIENT_H */
