@@ -1,4 +1,4 @@
-/*      $Id: tabwin.h 26999 2008-05-31 14:35:42Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
         MA 02110-1301, USA.
 
 
-        xfwm4    - (c) 2002-2008 Olivier Fourdan
+        xfwm4    - (c) 2002-2009 Olivier Fourdan
 
  */
 
@@ -32,33 +32,43 @@
 #include <gtk/gtk.h>
 
 typedef struct _Tabwin Tabwin;
+typedef struct _TabwinWidget TabwinWidget;
+
 struct _Tabwin
+{
+    GList *tabwin_list;
+    GList **client_list;
+    GList *selected;
+    gboolean display_workspace;
+};
+
+struct _TabwinWidget
 {
     /* The below must be freed when destroying */
     GtkWidget *window;
-    GList *head;
+    GList *widgets;
 
     /* these don't have to be */
+    Tabwin *tabwin;
     GtkWidget *class;
     GtkWidget *label;
     GtkWidget *container;
-    GList *current;
-    gulong selected_callback;
-    gboolean display_workspace;
+    GtkWidget *selected;
 
-    int grid_cols;
-    int grid_rows;
+    gulong selected_callback;
+    gint monitor_num;
+    gint width, height;
+    gint grid_cols;
+    gint grid_rows;
 };
 
-Tabwin                  *tabwinCreate                           (GdkScreen *,
-                                                                 Client *,
-                                                                 Client *,
-                                                                 unsigned int,
+Tabwin                  *tabwinCreate                           (GList **,
+                                                                 GList *,
                                                                  gboolean);
 Client                  *tabwinGetSelected                      (Tabwin *);
+Client                  *tabwinSelectHead                       (Tabwin *);
 Client                  *tabwinSelectNext                       (Tabwin *);
 Client                  *tabwinSelectPrev                       (Tabwin *);
-Client                  *tabwinGetHead                          (Tabwin *);
 Client                  *tabwinRemoveClient                     (Tabwin *,
                                                                  Client *);
 void                    tabwinDestroy                           (Tabwin *);
